import 'dart:async';

import 'package:flutter/foundation.dart';

import 'mobile_layer/ble_connectivity_mobile.dart';
import 'model/fhp_ble_charatereristic.dart';
import 'model/fhp_ble_device.dart';
import 'model/scan_properties.dart';
import 'model/sensor_data_result.dart';
import 'web_layer/ble_connectivity_web.dart';

abstract class BleConnectivityTargetPlatform {
  static BleConnectivityTargetPlatform _instance =
      BleConnectivityTargetPlatform();
  static BleConnectivityTargetPlatform get instance => _instance;

  Function(List<FhpBleDevice> sensors) scanResultsCallback;
  ScanProperties scanProperties;

  factory BleConnectivityTargetPlatform() {
    if (kIsWeb) {
      return BleConnectivityWeb();
    } else {
      return BleConnectivityMobile();
    }
  }

  initScan(ScanProperties scanProperties);

  disposeListeners();

  scanResults(Function(List<FhpBleDevice> sensors) result);

  Future<Stream<ScanEndReason>> startScan();

  stopScan();

  connectWithSensor(
    FhpBleDevice device,
    Function(BleDeviceState) connectionState,
  );

  disconnectWithSensor(FhpBleDevice device);

  Future<List<Stream<SensorDataResult>>> readSensorCharacteristics(
    FhpBleDevice device,
    Map<String, List<FhpBleCharacteristic>> characteristics,
  );

  Future<List<int>> readCharacteristic(FhpBleDevice device,
      {@required String serviceUuid, @required String characteristicUuid});
}

enum ScanEndReason { ended, canceled, error, bleOff }
