import 'package:ble_connectivity/iotip/iotip_constants.dart';
import 'package:ble_connectivity/model/ble_charge_state.dart';
import 'package:ble_connectivity/model/fhp_ble_device.dart';
import 'package:ble_connectivity/model/fhp_ble_device_status.dart';
import 'package:ble_connectivity/model/sensor_data_result.dart';
import 'package:ble_connectivity/sensor_abstraction_layer.dart';

class IotipConnectivity extends SensorAbstractionLayer {
  IotipConnectivity() : super(kIotipCharacteristics);

  @override
  processCharacteristicsEvent(
    SensorDataResult event,
    FhpBleDevice device,
    Function(FhpBleDevice) eventChanges,
  ) {
    if (event.characteristicUuid ==
        IotipCharateristic.batteryChargeState.uuid) {
      device.status.chargeState = event.value.length > 0
          ? event.value.first == 1
              ? BleChargeState.charging
              : BleChargeState.discharging
          : BleChargeState.unknown;

      eventChanges(device);
    } else if (event.characteristicUuid ==
            IotipCharateristic.batteryLevel.uuid ||
        event.characteristicUuid ==
            IotipCharateristic.currentBatteryLevel.uuid) {
      device.status.chargeValue =
          event.value.length > 0 ? event.value.first : -1;
      eventChanges(device);
    } else if (event.characteristicUuid ==
        IotipCharateristic.accelerometer.uuid) {
      var value = event.value;
      if (value.isEmpty) return;

      var sample = value.getRange(0, 10).toList();
      // var timestamp =
      //   (sample[0] | (sample[1] << 8) | (sample[2] << 16) | (sample[3] << 24))
      //     .unsignedRightShift(0);
      var x = (sample[4] | (sample[5] << 8)).toSigned(16) *
          ACCELEROMETER_UNIT_FACTOR;
      var y = (sample[6] | (sample[7] << 8)).toSigned(16) *
          ACCELEROMETER_UNIT_FACTOR;
      var z = (sample[8] | (sample[9] << 8)).toSigned(16) *
          ACCELEROMETER_UNIT_FACTOR;

      device.status.accelerometerData = AxisValue(x, y, z);

      sample = value.getRange(10, 20).toList();
      /*  timestamp = (sample[0] |
              (sample[1] << 8) |
              (sample[2] << 16) |
              (sample[3] << 24))
          .unsignedRightShift(0);*/
      x = (sample[4] | (sample[5] << 8)).toSigned(16) *
          ACCELEROMETER_UNIT_FACTOR;
      y = (sample[6] | (sample[7] << 8)).toSigned(16) *
          ACCELEROMETER_UNIT_FACTOR;
      z = (sample[8] | (sample[9] << 8)).toSigned(16) *
          ACCELEROMETER_UNIT_FACTOR;

      device.status.accelerometerData = AxisValue(x, y, z);
      eventChanges(device);
    }
  }
}
