import 'dart:async';

import 'package:ble_connectivity/ble_connectivity_target_platform.dart';
import 'package:ble_connectivity/model/fhp_ble_charatereristic.dart';
import 'package:ble_connectivity/model/fhp_ble_device.dart';
import 'package:ble_connectivity/model/sensor_data_result.dart';

abstract class SensorAbstractionLayer {
  final Map<String, List<FhpBleCharacteristic>> sensorCharacteristics;
  Map<FhpBleDevice, List<StreamSubscription>> _characteristicsStreams = {};

  /*late*/ BleConnectivityTargetPlatform blePlatform;

  SensorAbstractionLayer(this.sensorCharacteristics);

  readSensorCharacteristics(
    FhpBleDevice bleDevice,
    Map<String, List<FhpBleCharacteristic>> characteristics,
    Function(FhpBleDevice) eventChanges,
  ) async {
    if (!bleDevice.isConnected) return;

    if (this._characteristicsStreams[bleDevice] == null) {
      this._characteristicsStreams[bleDevice] = [];
      List<Stream<SensorDataResult>> _characteristicsStreams =
          await blePlatform.readSensorCharacteristics(
        bleDevice,
        characteristics,
      );

      _characteristicsStreams.forEach((it) {
        this._characteristicsStreams[bleDevice].add(
          it.listen((event) {
            processCharacteristicsEvent(event, bleDevice, eventChanges);
          }),
        );
      });
    } else {
      this._characteristicsStreams[bleDevice]?.forEach((element) {
        element.resume();
      });
    }

    /*  if (!kIsWeb) {
      List<int> batteryLevel = await BleConnectivity.instance
          .readCharacteristic(bleDevice,
              serviceUuid: IotipService.battery.uuid,
              characteristicUuid: IotipCharateristic.batteryLevel.uuid);

      _processCharacteristicEvent(
          SensorDataResult(
              value: batteryLevel,
              characteristicUuid: IotipCharateristic.batteryLevel.uuid,
              device: bleDevice),
          bleDevice);
    }*/
  }

  processCharacteristicsEvent(
    SensorDataResult event,
    FhpBleDevice device,
    Function(FhpBleDevice) eventChanges,
  );

  void disconnectWithSensor(FhpBleDevice device) {
    _characteristicsStreams[device]?.forEach((element) {
      element.cancel();
    });
    _characteristicsStreams[device] = null;
  }
}
