import 'dart:async';
// In order to *not* need this ignore, consider extracting the "web" version
// of your plugin as a separate package, instead of inlining it in the same
// package as the core of your plugin.
// ignore: avoid_web_libraries_in_flutter
import 'dart:html' as html show window;

import 'package:ble_connectivity/web_layer/javascript_layer.dart';
import 'package:flutter/services.dart';
import 'package:flutter_web_plugins/flutter_web_plugins.dart';
import 'package:import_js_library/import_js_library.dart';

import 'channel_methods.dart';

/// A web implementation of the BleConnectivity plugin.

class BleWebChannel {
  static void registerWith(Registrar registrar) {
    final MethodChannel channel = MethodChannel(
      'ble_connectivity',
      const StandardMethodCodec(),
      registrar.messenger,
    );

    importJsLibrary(
        url: "./web/interop.js", flutterPluginName: "ble_connectivity");
    importJsLibrary(
        url: "./web/sensors/ble_device.js",
        flutterPluginName: "ble_connectivity");
    importJsLibrary(
        url: "./web/sensors/sensors.js", flutterPluginName: "ble_connectivity");
    importJsLibrary(
        url: "./web/sensors/utils.js", flutterPluginName: "ble_connectivity");

    final pluginInstance = BleWebChannel();
    channel.setMethodCallHandler(pluginInstance.handleMethodCall);
  }

  /// Handles method calls over the MethodChannel of this plugin.
  /// Note: Check the "federated" architecture for a new way of doing this:
  /// https://flutter.dev/go/federated-plugins
  Future<dynamic> handleMethodCall(MethodCall call) async {
    print('handleMethodCall');
    switch (call.method) {
      case 'getPlatformVersion':
        return getPlatformVersion();
        break;
      case ChannelMethods.kInitProperties:
        print(ChannelMethods.kInitProperties);
        return setProperties();
        break;
      case ChannelMethods.kTryConnect:
        print('tryConnect with ${call.arguments['prefix']}');
        return tryConnectJS(call.arguments['prefix'],
            call.arguments['scanDuration'], call.arguments['characteristics']);
        break;

      case ChannelMethods.kReadSensorCharacteristics:
        print('ChannelMethods.kReadSensorCharacteristics');
        return readSensorCharacteristicsJS(
            call.arguments['device'], call.arguments['characteristics']);
        break;
      case ChannelMethods.kDisconnectSensor:
        print('ChannelMethods.kDisconnectSensor');
        return disconnectSensorJS(call.arguments['device']);
        break;
      case ChannelMethods.kReadCurrentValues:
        print('ChannelMethods.kReadCurrentValues');
        return onCurrentDeviceValueJS(
          call.arguments['device'],
          call.arguments['service'],
          call.arguments['characteristic'],
        );
        break;
      default:
        throw PlatformException(
          code: 'Unimplemented',
          details:
              'ble_connectivity for web doesn\'t implement \'${call.method}\'',
        );
    }
  }

  /// Returns a [String] containing the version of the platform.
  Future<String> getPlatformVersion() {
    final version = html.window.navigator.userAgent;
    return Future.value(version);
  }
}
